package com.ruoyi.web.controller.app;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.RlUser;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.wx.vo.PrepayResponseVO;
import com.ruoyi.system.domain.device.RlDevice;
import com.ruoyi.system.domain.device.RlDeviceQuery;
import com.ruoyi.system.domain.device.RlDeviceVO;
import com.ruoyi.system.domain.deviceAccessory.RlDeviceAccessory;
import com.ruoyi.system.domain.order.RlOrderQuery;
import com.ruoyi.system.domain.order.RlOrderVO;
import com.ruoyi.system.domain.query.AuthenticationQuery;
import com.ruoyi.system.service.IRlDeviceAccessoryService;
import com.ruoyi.system.service.IRlDeviceService;
import com.ruoyi.system.service.IRlOrderService;
import com.ruoyi.system.service.IRlUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

/**
 * app接口（需要登录校验的）
 *  校验
 * @author ruoyi
 */
@RestController
@RequestMapping("/appVerify")
public class AppVerifyController extends BaseController
{

    @Resource
    private IRlUserService rlUserService;

    @Resource
    private IRlOrderService orderService;

    @Autowired
    private IRlDeviceService deviceService;

    @Autowired
    private IRlDeviceAccessoryService deviceAccessoryService;

    /**
     * 实名认证
     */
    @Log(title = "实名认证", businessType = BusinessType.AUTHENTICATION)
    @GetMapping("/user/authentication")
    public AjaxResult authentication(AuthenticationQuery authenticationQuery)
    {
        logger.info("【实名认证】请求参数：authenticationQuery={}", JSON.toJSON(authenticationQuery));
        Object authentication = rlUserService.authentication(authenticationQuery);
        //判断o是否是字符串
        if(authentication instanceof String){
            return AjaxResult.error((String) authentication);
        }else{
            return AjaxResult.success(authentication);
        }
    }

    /**
     * 下单
     */
    @Log(title = "下单", businessType = BusinessType.AUTHENTICATION)
    @PostMapping("/createOrder")
    public AjaxResult createOrder(@RequestBody RlOrderQuery order)
    {
        logger.info("【下单】请求参数：order={}", JSON.toJSON(order));
        PrepayResponseVO orderServiceOrder = orderService.createOrder(order);
        return success(orderServiceOrder);
    }

    /**
     * 订单列表
     */
    @GetMapping("/orderList")
    public AjaxResult orderList(RlOrderQuery order)
    {
        logger.info("订单列表请求：order=【"+JSON.toJSONString(order)+"】");
        List<RlOrderVO> rlOrderVOS = orderService.selectRlOrderList(order);
        return success(rlOrderVOS);
    }

    /**
     * 订单信息
     */
    @GetMapping("/orderInfo")
    public AjaxResult orderInfo(String orderNo)
    {
        logger.info("订单列表请求：order=【"+JSON.toJSON(orderNo)+"】");
        RlOrderVO rlOrder = orderService.getOrderInfoByOrderNo(orderNo);
        return success(rlOrder);
    }

    /**
     * 扫码绑定车辆
     */
    @Log(title = "扫码绑定车辆", businessType = BusinessType.BANDDEVICE)
    @PostMapping("/bandDevice")
    public AjaxResult bandDevice(String orderNo, String sn)
    {
        logger.info("扫码绑定车辆：orderNo=【"+orderNo+"】,sn=【"+sn+"】");
        orderService.bandDevice(orderNo,sn);
        return success();
    }

    /**
     * 临时锁车
     */
    @Log(title = "临时锁车", businessType = BusinessType.LOCK)
    @PostMapping("/device/lock")
    public AjaxResult deviceLock(String sn,String orderNo,String isBluetooth,String lon,String lat)
    {
        logger.info("【临时锁车请求参数】：{},订单号：{},是否蓝牙控制：{}", sn,orderNo,isBluetooth);
        Boolean i =deviceService.lock(sn,orderNo,isBluetooth,lon,lat);
        return success(i);
    }

    /**
     * 临时解锁
     */
    @Log(title = "临时解锁", businessType = BusinessType.UNLOCK)
    @PostMapping("/device/unlock")
    public AjaxResult deviceUnlock(String sn,String orderNo,String isBluetooth,String lon,String lat)
    {
        logger.info("【临时解锁请求参数】：{},订单号：{},是否蓝牙控制：{}", sn,orderNo,isBluetooth);
        Boolean i =deviceService.unlock(sn,orderNo,isBluetooth,lon,lat);
        return success(i);
    }

    /**
     * 更新车牌号
     */
    @Log(title = "更新车牌号", businessType = BusinessType.UPDATE)
    @PutMapping("/device/edit")
    public AjaxResult deviceEdit(String sn, String vehicleNum, String status)
    {
        RlDevice device = new RlDevice();
        device.setSn(sn);
        device.setVehicleNum(vehicleNum);
        device.setStatus(status);
        return toAjax(deviceService.updateDeviceBySn(device));
    }


    /**
     * 续租
     */
    @Log(title = "续租", businessType = BusinessType.RELET)
    @PostMapping("/relet")
    public AjaxResult relet(String orderNo,Long ruleId )
    {
        logger.info("【续租】请求参数：orderNo={},ruleId={}", orderNo,ruleId);
        PrepayResponseVO responseVO =orderService.relet(orderNo,ruleId);
        return success(responseVO);
    }

    /**
     * 还车
     */
    @Log(title = "还车", businessType = BusinessType.RETURN_VEHICLE)
    @PostMapping("/returnVehicle")
    public AjaxResult returnVehicle(@RequestBody RlOrderQuery order)
    {
        logger.info("【还车】请求参数：order={}", JSON.toJSONString(order));
        Boolean aBoolean = deviceService.returnVehicle(order);
        return success(aBoolean);
    }

    /**
     * 订单取消
     */
    @Log(title = "订单取消", businessType = BusinessType.ORDERCANCEL)
    @PostMapping("/orderCancel")
    public AjaxResult orderCancel(String orderNo)
    {
        logger.info("【订单取消】请求参数：orderNo={}", orderNo);
        Boolean aBoolean = orderService.orderCancel(orderNo);
        return success(aBoolean);
    }

    /**
     * 车辆列表
     */
    @GetMapping(value = "/vehicleList")
    public AjaxResult allVehicleInfo(RlDeviceQuery query)
    {
        logger.info("【车辆列表】请求参数：query={}", JSON.toJSONString(query));
        List<RlDeviceVO> devices = deviceService.selectDeviceList(query);
        return success(devices);
    }

    /**
     * 管理员开锁
     */
    @Log(title = "管理员开锁", businessType = BusinessType.UNLOCKING)
    @PostMapping("/admin/unlocking")
    public AjaxResult unlocking(String sn)
    {
        logger.info("【管理员开锁请求】：{}",sn);
        RlUser rlUser = getLoginUser().getRlUser();
        Boolean aBoolean = deviceService.unlocking(sn,rlUser.getUserName());
        return success(aBoolean);
    }

    /**
     * 管理员关锁
     */
    @Log(title = "管理员关锁", businessType = BusinessType.LOCK)
    @PostMapping("/admin/lock")
    public AjaxResult lock(String sn)
    {
        logger.info("【管理员关锁请求】：{}",sn);
        RlUser rlUser = getLoginUser().getRlUser();
        Boolean aBoolean = deviceService.adminLock(sn,rlUser.getUserName());
        return success(aBoolean);
    }

    /**
     * 选择配件
     */
    @Log(title = "选择配件", businessType = BusinessType.LOCK)
    @PostMapping("/select/accessory")
    public AjaxResult lock(Long deviceId,Long[] accessoryIds)
    {
        if(deviceId==null){
            return error("【选择配件】车辆id不能为空");
        }
        if(accessoryIds==null||accessoryIds.length==0){
            return error("【选择配件】配件id不能为空");
        }
        logger.info("【选择配件】请求参数：deviceId={},accessoryIds={}", deviceId,accessoryIds);
        int i = deviceAccessoryService.deleteRlDeviceAccessoryByDeviceId(deviceId);
        if(i>0){
            logger.info("【删除成功】");
            for (Long accessoryId:accessoryIds) {
                RlDeviceAccessory rlDeviceAccessory = new RlDeviceAccessory();
                rlDeviceAccessory.setAccessoryId(accessoryId);
                rlDeviceAccessory.setDeviceId(deviceId);
                deviceAccessoryService.insertRlDeviceAccessory(rlDeviceAccessory);
            }
            return success();
        }
        return error();
    }

    /**
     * 车损扣款
     */
    @Log(title = "车损扣款", businessType = BusinessType.DEDUCTMONEY)
    @PostMapping("/deductMoney")
    public AjaxResult deductMoney(BigDecimal money,String orderNo)
    {
        logger.info("【车损扣款】请求参数：money={},orderNo={}", money,orderNo);
        if(money==null){
            return error("扣款金额不能为空");
        }
        if(money.compareTo(BigDecimal.ZERO)<=0){
            return error("扣款金额不能为0");
        }
        if(StrUtil.isEmpty(orderNo)){
            return error("订单号不能为空");
        }
        Boolean aBoolean = orderService.deductMoney(money,orderNo);
        if(aBoolean){
            return success();
        }
        return error();
    }

    /**
     * 押金退款
     */
    @Log(title = "押金退款", businessType = BusinessType.DEPOSITREFUND)
    @PostMapping("/depositRefund")
    public AjaxResult depositRefund(String orderNo)
    {
        logger.info("【押金退款】请求参数：orderNo={}", orderNo);
        if(StrUtil.isEmpty(orderNo)){
            return error("订单号不能为空");
        }
        Boolean aBoolean = orderService.depositRefund(orderNo);
        if(aBoolean){
            return success();
        }
        return error();
    }

}
