package com.ruoyi.system.service;

import com.ruoyi.common.utils.wx.vo.PrepayResponseVO;
import com.ruoyi.system.domain.order.RlOrder;
import com.ruoyi.system.domain.order.RlOrderQuery;
import com.ruoyi.system.domain.order.RlOrderVO;

import java.math.BigDecimal;
import java.util.List;

/**
 * 订单Service接口
 *
 * @author qzz
 * @date 2024-09-19
 */
public interface IRlOrderService
{
    /**
     * 查询订单
     *
     * @param orderId 订单主键
     * @return 订单
     */
    public RlOrderVO selectRlOrderByOrderId(Long orderId);

    /**
     * 查询订单
     *
     * @param orderNo 订单主键
     * @return 订单
     */
    public RlOrderVO selectRlOrderByOrderNo(String orderNo);

    /**
     * 查询订单详情，详细信息
     *
     * @param orderNo 订单主键
     * @return 订单
     */
    public RlOrderVO getOrderInfoByOrderNo(String orderNo);

    /**
     * 查询订单列表
     *
     * @param rlOrder 订单
     * @return 订单集合
     */
    public List<RlOrderVO> selectRlOrderList(RlOrderQuery rlOrder);

    /**
     * 新增订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    public int insertRlOrder(RlOrderQuery rlOrder);

    /**
     * 修改订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    public int updateRlOrder(RlOrder rlOrder);

    /**
     * 批量删除订单
     *
     * @param orderIds 需要删除的订单主键集合
     * @return 结果
     */
    public int deleteRlOrderByOrderIds(Long[] orderIds);

    /**
     * 删除订单信息
     *
     * @param orderId 订单主键
     * @return 结果
     */
    public int deleteRlOrderByOrderId(Long orderId);

    /**
     * 下单
     */
    PrepayResponseVO createOrder(RlOrderQuery order);

    /**
     * 扫码绑定车辆
     */
    int bandDevice(String orderNo, String sn);

    /**
     * 续租
     */
    PrepayResponseVO relet(String orderNo, Long ruleId);

    /**
     * 订单取消
     */
    Boolean orderCancel(String orderNo);

    /**
     * 车损扣款
     */
    Boolean deductMoney(BigDecimal money, String orderNo);

    /**
     * 押金退款
     */
    Boolean depositRefund(String orderNo);
}
