package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.common.utils.wx.vo.PrepayResponseVO;
import com.ruoyi.system.domain.accessory.RlAccessoryVO;
import com.ruoyi.system.domain.device.RlDevice;
import com.ruoyi.system.domain.order.RlOrder;
import com.ruoyi.system.domain.order.RlOrderQuery;
import com.ruoyi.system.domain.order.RlOrderVO;
import com.ruoyi.system.domain.rule.RlFeeRule;
import com.ruoyi.system.mapper.RlOrderMapper;
import com.ruoyi.system.service.*;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 订单Service业务层处理
 *
 * @author qzz
 * @date 2024-09-19
 */
@Service
public class RlOrderServiceImpl implements IRlOrderService
{
    @Resource
    private RlOrderMapper orderMapper;

    @Resource
    private IWxPayService wxPayService;

    @Autowired
    private IRlFeeRuleService feeRuleService;

    @Resource
    private IRlCityService cityService;

    @Autowired
    private IRlAccessoryService accessoryService;

    @Autowired
    private IRlDeviceService deviceService;

    /**
     * 查询订单
     *
     * @param orderId 订单主键
     * @return 订单
     */
    @Override
    public RlOrderVO selectRlOrderByOrderId(Long orderId)
    {
        return orderMapper.selectRlOrderByOrderId(orderId);
    }

    /**
     * 查询订单
     *
     * @param orderNo 订单号
     * @return 订单
     */
    @Override
    public RlOrderVO selectRlOrderByOrderNo(String orderNo)
    {
        return orderMapper.selectRlOrderByOrderNo(orderNo);
    }

    /**
     * 查询订单
     *
     * @param orderNo 订单号
     * @return 订单
     */
    @Override
    public RlOrderVO getOrderInfoByOrderNo(String orderNo)
    {
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        RlDevice rlDevice = deviceService.selectDeviceBySn(order.getSn());
        if(ObjectUtil.isNull(rlDevice)) throw new RuntimeException("未找到车辆信息");
        List<RlAccessoryVO> accessorys = accessoryService.selectRlDeviceAccessoryByDeviceId(rlDevice.getDeviceId());
        order.setAccessorys(accessorys);
        order.setPicture(rlDevice.getPicture());
        return order;
    }

    /**
     * 查询订单列表
     *
     * @param rlOrder 订单
     * @return 订单
     */
    @Override
    public List<RlOrderVO> selectRlOrderList(RlOrderQuery rlOrder)
    {
        return orderMapper.selectRlOrderList(rlOrder);
    }

    /**
     * 新增订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    @Override
    public int insertRlOrder(RlOrderQuery rlOrder)
    {
        rlOrder.setCreateTime(DateUtils.getNowDate());
        return orderMapper.insertRlOrder(rlOrder);
    }

    /**
     * 修改订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    @Override
    public int updateRlOrder(RlOrder rlOrder)
    {
        return orderMapper.updateRlOrder(rlOrder);
    }

    /**
     * 批量删除订单
     *
     * @param orderIds 需要删除的订单主键
     * @return 结果
     */
    @Override
    public int deleteRlOrderByOrderIds(Long[] orderIds)
    {
        return orderMapper.deleteRlOrderByOrderIds(orderIds);
    }

    /**
     * 删除订单信息
     *
     * @param orderId 订单主键
     * @return 结果
     */
    @Override
    public int deleteRlOrderByOrderId(Long orderId)
    {
        return orderMapper.deleteRlOrderByOrderId(orderId);
    }

    /**
     * 下单
     * 1. 创建订单
     * 2. 预支付
     * 3.
     *
     * @param order 订单对象
     * @return 结果
     */
    @Override
    public PrepayResponseVO createOrder(RlOrderQuery order) {
        String orderNo = IdUtils.getOrderNo("zl");
        PrepayResponseVO responseVO = getPrepayResponseVO(order,orderNo,ServiceConstants.ORDER_TYPE_LEASE);
        return responseVO;
    }

    @NotNull
    private PrepayResponseVO getPrepayResponseVO(RlOrderQuery order,String orderNo, String type) {
        RlFeeRule feeRule = feeRuleService.selectRlFeeRuleByRuleId(order.getRuleId());
        setFeeRule(order, feeRule);
        /** 解析地址  analyzeAddress */
        analyzeAddress(order);
        order.setCreateTime(DateUtils.getNowDate());
        order.setOrderNo(orderNo);
        order.setPaid(ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT);
        order.setType(type);
        Integer deliveryMethod = order.getDeliveryMethod();
        if(ServiceConstants.DELIVERY_METHOD_SELF_PICKUP == deliveryMethod){
            order.setStatus(ServiceConstants.ORDER_STATUS_TO_BE_TAKEN);
        } else if(ServiceConstants.DELIVERY_METHOD_FREE_DELIVERY_CAR == deliveryMethod || ServiceConstants.DELIVERY_METHOD_CHARGE_DELIVERY_CAR == deliveryMethod){
            order.setStatus(ServiceConstants.ORDER_STATUS_TO_BE_SENT);
        } else {
            throw new RuntimeException("暂不支持该配送方式");
        }
        int i = orderMapper.insertRlOrder(order);
        if (i < 1) {
           throw new RuntimeException("下单失败");
        }
        PrepayWithRequestPaymentResponse response = wxPayService.prepayWithRequestPayment(order);
        PrepayResponseVO responseVO = new PrepayResponseVO();
        responseVO.setResponse(response);
        return responseVO;
    }

    /** 解析地址 */
    private void analyzeAddress(RlOrderQuery order) {
        String address = cityService.getAddressByLocation(order.getPickupLon(),order.getPickupLat());
        order.setPickupLoc(address);
    }

    private void setFeeRule(RlOrderQuery order, RlFeeRule feeRule) {
        order.setNum(feeRule.getNum());
        order.setPrice(feeRule.getPrice());
        order.setExplain(feeRule.getExplain());
        order.setInstructions(feeRule.getInstructions());
        order.setOutUnit(feeRule.getOutUnit());
        order.setOutPrice(feeRule.getOutPrice());
        order.setRentalUnit(feeRule.getRentalUnit());
        /** 如果是租赁订单，根据取车时间计算到期时间
         * 如果是续费订单，根据到期时间计算续费订单的到期时间 */
        Date startTime = null;
        if(order.getType().equals(ServiceConstants.ORDER_TYPE_LEASE)){
            startTime = order.getPickupTime();
        }else{
            startTime = order.getExpiryTime();
        }
        Date expiryTime = DateUtils.getTimeAfterXUnit(startTime, feeRule.getNum(), feeRule.getRentalUnit());
        order.setExpiryTime(expiryTime);
    }

    /**
     * 扫码绑定车辆
     */
    @Override
    public int bandDevice(String orderNo, String sn) {
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(order == null){
            throw new RuntimeException("订单不存在");
        }
        if(StrUtil.isNotBlank(order.getSn())){
            throw new RuntimeException("该订单已绑定车辆："+order.getSn());
        }
        RlOrder rlOrder = new RlOrder();
        rlOrder.setOrderNo(orderNo);
        rlOrder.setSn(sn);
        return orderMapper.updateRlOrderByOrderNo(rlOrder);
    }

    /**
     * 续租
     * 到期时间
     * 1.  根据原来的订单号，写入新的订单
     * 2.  续租订单标记类型
     * 3.
     */
    @Override
    public PrepayResponseVO relet(String orderNo, Long ruleId) {
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        RlOrderQuery orderQuery = new RlOrderQuery();
        BeanUtils.copyBeanProp(orderQuery,order);
        /** 原订单已经到期 */
        Date nowDate = DateUtils.getNowDate();
        if (order.getExpiryTime().before(nowDate)) {
            throw new RuntimeException("订单已过期，请重新下单");
        }
        orderQuery.setOriginalOrderNo(orderNo);
        orderQuery.setRuleId(ruleId);
        orderQuery.setPickupTime(order.getExpiryTime());
        String newOrderNo = IdUtils.getOrderNo("xz");
        PrepayResponseVO responseVO = getPrepayResponseVO(orderQuery,newOrderNo,ServiceConstants.ORDER_TYPE_RELET);
        return responseVO;
    }

    /**
     * 订单取消
     *
     */
    @Override
    public Boolean orderCancel(String orderNo) {
        if(StrUtil.isBlank(orderNo)){
            throw new RuntimeException("订单号不能为空");
        }
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(order == null){
            throw new RuntimeException("订单不存在");
        }
        if(!(ServiceConstants.ORDER_STATUS_TO_BE_TAKEN.equals(order.getStatus()) || ServiceConstants.ORDER_STATUS_TO_BE_SENT.equals(order.getStatus()))){
            throw new RuntimeException("订单已配送或已使用，不能取消");
        }
        if(ServiceConstants.ORDER_PAY_STATUS_PAID.equals(order.getPaid())){
            throw new RuntimeException("订单已支付，不能取消");
        }
        RlOrder rlOrder = new RlOrder();
        rlOrder.setOrderNo(orderNo);
        rlOrder.setPaid(ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT);
        rlOrder.setStatus(ServiceConstants.ORDER_STATUS_CANCE);
        int i = orderMapper.updateRlOrderByOrderNo(rlOrder);
        if(i>0){
            return true;
        }
        return false;
    }

    /**
     * 车损扣款
     */
    @Override
    public Boolean deductMoney(BigDecimal money, String orderNo) {
        RlOrder rlOrder = new RlOrder();
        rlOrder.setOrderNo(orderNo);
        rlOrder.setDeductionAmount(money);
        int i = orderMapper.updateRlOrderByOrderNo(rlOrder);
        return i>0;
    }

    /**
     * 押金退款
     */
    @Override
    public Boolean depositRefund(String orderNo) {
        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        BigDecimal deposit = order.getDeposit();
        BigDecimal overdueFee = order.getOverdueFee();
        BigDecimal remainingDeposit = deposit.subtract(overdueFee);
        if(remainingDeposit.compareTo(BigDecimal.ZERO) <= 0){
            return true;
        }
        if(!ServiceConstants.ORDER_PAY_STATUS_PAID.equals(order.getPaid())){
          throw new ServiceException("订单未支付，不能退款", HttpStatus.ERROR);
        }
        if(!ServiceConstants.ORDER_STATUS_ORDER_END.equals(order.getStatus())){
            throw new ServiceException("订单状态异常，不能退款", HttpStatus.ERROR);
        }
        // 执行退还剩余押金操作
        wxPayService.refund(order, "审核后退押金",remainingDeposit,order.getOutTradeNo());
        return null;
    }


}
